package turgen;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.Window;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.HashMap;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import turgen.config.AbstractConfigEntry;
import turgen.config.ConfigDialog;
import turgen.config.ConfigEntryListener;
import turgen.config.StringConfigEntry;
import turgen.dtb.StatisticsDialog;
import turgen.wizard.WizardDialog;

/**
 * Centralized Frame and Dialog handling
 *
 */
public class DialogManager implements ConfigEntryListener {
    
    /**Main window*/
    public TurgenFrame frmTurgen;

    /**Dialog used for editing playlist items*/
    PlayListItemDialog dlgRecord;
    
    /**Frame for DOS 2 binaries merging*/
    MergerDialog dlgMergeBinary;

    /**Dialog for configuration*/
    ConfigDialog dlgConfig;
    
    /**Frame for turbo decoder*/
    DecoderFrame frmDecoder;

    /**Statistics dialog*/
    public StatisticsDialog dlgStatistics;

    /**Wizard frame*/
    public WizardDialog dlgWizard;
    
    public EmbedBasicDialog dlgEmbedBasic;
    
   
    /**Configuration of look and feel*/
    StringConfigEntry cfgLookAndFeel;
    
    /**Last look and feel set*/
    private String lastLookAndFeel="";

    private UIPersistor[] windowArray;


    
    private DialogManager() {
        /*Look and feel*/
        String[] plafs = new String[6];
        plafs[0]="javax.swing.plaf.metal.MetalLookAndFeel";
        plafs[1]="com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel";
        plafs[2]="com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
        plafs[3]="com.sun.java.swing.plaf.motif.MotifLookAndFeel";
        plafs[4]="com.sun.java.swing.plaf.gtk.GTKLookAndFeel";
        plafs[5]="com.sun.java.swing.plaf.mac.MacLookAndFeel";
        cfgLookAndFeel = new StringConfigEntry("lookAndFeel","GUI Look and feel","",plafs,true);
        Turgen.cfcTurgen.addEntry(cfgLookAndFeel);
    }
    
    public static DialogManager getInstance() {
        return new DialogManager();
    }
    
    
    /** Create all dialogs*/
    public void initDialogs() {

        /*Initializing particular dialogs*/
        frmTurgen=new TurgenFrame();
        dlgRecord = new PlayListItemDialog(frmTurgen);
        dlgMergeBinary = new MergerDialog();
        dlgConfig = new ConfigDialog(frmTurgen);
        frmDecoder = new DecoderFrame();
        dlgStatistics = new StatisticsDialog(dlgRecord);
        dlgWizard = new WizardDialog();
        dlgEmbedBasic = new EmbedBasicDialog();

        windowArray = new UIPersistor[8];
        
        windowArray[0]=frmTurgen;
        windowArray[1]=dlgRecord;
        windowArray[2]=dlgMergeBinary;
        windowArray[3]=dlgConfig;
        windowArray[4]=frmDecoder;
        windowArray[5]=dlgStatistics;
        windowArray[6]=dlgWizard;
        windowArray[7]=dlgEmbedBasic;
        
        cfgLookAndFeel.setListener(this);
        
    }
    
    /** Restore frame/dialog bounds. If restoration not possible, call
     *  pack and center them
     */
    void loadBounds() {
        
        FileInputStream fis;
        DataInputStream dis;
        ObjectInputStream ois=null;
        HashMap uiPersistenceMap = new HashMap();
        
        try {

            fis = new FileInputStream(Turgen.configDir+"winlayout.ser");
            dis = new DataInputStream(fis);
            
            int numObjects = dis.readInt();
            
            for(int i=0;i<numObjects;i++) {
                
                    String key = dis.readUTF();
                    int objectLength = dis.readInt();
                    byte[] objectBytes = new byte[objectLength];
                    dis.read(objectBytes);
                    
                    try {
                        ByteArrayInputStream bais = new ByteArrayInputStream(objectBytes);
                        ois = new ObjectInputStream(bais);
                        Object value = ois.readObject();
                        uiPersistenceMap.put(key,value);
                        ois.close();
                    }
                    catch(Exception ex) {
                        for(int k=0;k<windowArray.length;k++) {
                            if (((UIPersistor)windowArray[k]).getPersistenceId().equals(key)) {
                                applyDefaultBounds(windowArray[k]);
                                uiPersistenceMap.put(key,windowArray[k].getPersistenceData());
                                break;
                            }
                        }
                        ex.printStackTrace();
                        ois.close();
                        
                    }
                    
            }
            
            
        }
        catch (Exception e) {

            e.printStackTrace();
            for(int i=0;i<windowArray.length;i++) {
                applyDefaultBounds(windowArray[i]);
            }
            
            return;
      }

        for(int i=0;i<windowArray.length;i++) {
            try {
                windowArray[i].setPersistenceData((Object[])uiPersistenceMap.get(windowArray[i].getPersistenceId()));
            }
            catch(Exception e) {
                e.printStackTrace();
                windowArray[i].setPersistenceDefaults();
            }
        }
        
    }
    /** Pokusi se ulozit pozice dialogu do souboru*/
    void saveBounds() {

        FileOutputStream fos;
        DataOutputStream dos;
        
        try {
            fos = new FileOutputStream(Turgen.configDir+"winlayout.ser");
            dos = new DataOutputStream(fos);

            HashMap uiPersistenceMap = new HashMap();
            
            /*First number of objects*/
            dos.writeInt(windowArray.length);
            
            for(int i=0;i<windowArray.length;i++) {
                
                Object value = windowArray[i].getPersistenceData();
                
                /*Temporary stream*/
                ByteArrayOutputStream baos = new ByteArrayOutputStream(64);
                ObjectOutputStream toos = new ObjectOutputStream(baos);
                toos.writeObject(value);
                toos.flush();
                toos.close();
                byte[] objectBytes = baos.toByteArray();
                
                /*Write key, length and value*/
                String key = windowArray[i].getPersistenceId();
                dos.writeUTF(key);
                dos.writeInt(objectBytes.length);
                dos.write(objectBytes);
                uiPersistenceMap.put(key,value);
            }
            
        }
        catch(Exception e) {
            e.printStackTrace();
        }
      
        
    }
    
    /**Center container 
     * @param c Container to center*/
    public static void centerContainer(Container c) {
        Dimension _d =Toolkit.getDefaultToolkit().getScreenSize();
    	int _x,_y;
    	_x = (_d.width-c.getBounds().width)/2;   
	_y = (_d.height-c.getBounds().height)/2;  
	c.setLocation(_x,_y);
    }
    
    
    public void onConfigEntryChange(AbstractConfigEntry entry) {
        final AbstractConfigEntry ace = entry;
        if (SwingUtilities.isEventDispatchThread()) {
            try {
                SwingUtilities.invokeLater(new Runnable() {
                   
                    public void run() {
                        onConfigEntryChangeImpl(ace);
                    }
                });
            }
            catch(Exception e) {
                e.printStackTrace();
            }
        }
        else {
            try {
                SwingUtilities.invokeAndWait(new Runnable() {
                   
                    public void run() {
                        onConfigEntryChangeImpl(ace);
                    }
                });
            }
            catch(Exception e) {
                e.printStackTrace();
            }
        }
    }
    
    public void onConfigEntryChangeImpl(AbstractConfigEntry entry)  {
        
        /*Pick class name and check if it is blank*/
        StringConfigEntry sce = (StringConfigEntry)entry;
        String plafClass = sce.getAsString();
        if (sce.isBlank() == true) {
            plafClass = System.getProperty("swing.defaultlaf");
            if (plafClass==null) return;
        }
        
        /*Change look and feel*/
        if (!lastLookAndFeel.equals(plafClass)) {
            lastLookAndFeel = plafClass;

            try {
                UIManager.setLookAndFeel(plafClass);
                SwingUtilities.updateComponentTreeUI(frmTurgen);
                SwingUtilities.updateComponentTreeUI(frmDecoder);
                SwingUtilities.updateComponentTreeUI(dlgRecord);
                SwingUtilities.updateComponentTreeUI(dlgMergeBinary);
                SwingUtilities.updateComponentTreeUI(dlgRecord);
                SwingUtilities.updateComponentTreeUI(dlgConfig);
                SwingUtilities.updateComponentTreeUI(dlgStatistics);
                SwingUtilities.updateComponentTreeUI(dlgWizard);
                SwingUtilities.updateComponentTreeUI(dlgEmbedBasic);
                
                frmTurgen.updateFileDialogUI();
                frmDecoder.updateFileDialogUI();
                dlgRecord.updateFileDialogUI();
                dlgMergeBinary.updateFileDialogUI();
                dlgWizard.updateFileDialogUI();
                dlgEmbedBasic.updateFileDialogUI();
                

            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        
    }
    
    public void applyDefaultBounds(UIPersistor window) {
        final UIPersistor windowImpl = window;
        if (SwingUtilities.isEventDispatchThread()) {
            try {
                SwingUtilities.invokeLater(new Runnable() {
                   
                    public void run() {
                        applyDefaultBoundsImpl(windowImpl);
                    }
                });
            }
            catch(Exception e) {
                e.printStackTrace();
            }
        }
        else {
            try {
                SwingUtilities.invokeAndWait(new Runnable() {
                   
                    public void run() {
                        applyDefaultBoundsImpl(windowImpl);
                    }
                });
            }
            catch(Exception e) {
                e.printStackTrace();
            }
        }
    }
    
    public void applyDefaultBoundsImpl(UIPersistor window) {
        centerContainer((Container)window);
        window.setPersistenceDefaults();
    }
}


